/*! \file GeneR_glob.h
 *  \brief Global variables of GeneR (i.e sequences buffers)
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 19:28:06 antoine>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

/*void init_GeneR();*/


#ifndef _GENER_GLOB_INCLUDED
#define _GENER_GLOB_INCLUDED 1

#include <vector>
class GeneR_seq;
//#include "GeneR_seq.h"


extern "C"
{
  void R_initialise_GeneR(int * maxbuffers);
  void myGeneR_alloc (int *size, int *bufno, int *fill, int *comp,int * err);
  void size_buf (int  *bufno,int * comp);
  void sys_maxbuffer(int * nb);
  void getParam(int * bufno, int * beg, int * size, int * masterSize, int * err);
  void setParam(int * bufno, int * beg, int * masterSize, int * err);
  void freeSeq(int * bufno);
  void getAccno (int * bufno,char ** name,int * err);
  void setAccno (int * bufno,char ** name,int * err);
  void getStrand (int * strandR);
  void setStrand (int * strandR);

}

class GeneR_glob
{
 public:

  GeneR_glob();
  GeneR_glob(const GeneR_glob & _src);

  ~GeneR_glob();

  int allocBuffer(int size, int bufno, int comp, int fill);

  char * buffer(int bufno,int comp=0);

  /** \fn size(int bufno,int comp=0)
   * \brief returns size of a buffer. Returns -1 if buffer not in range
   * \param bufno buffer number 
   * \param comp 0: forward, 1 reverse.
   * \note size of buffer uninitialized is 0;
   */
  int size(int bufno,int comp=0);
  int maxBuffers();
  void freeSeq(int  bufno);
  void freeCompSeq(int  bufno);

  void init(int maxbuffers_p);  


  void getAccno (int  bufno,char * name,int * err);
  void setAccno (int  bufno,char * name,int * err);

  int setParam(int bufno, int beg, int  masterSize);
  int getParam(int bufno, int & beg, int & size, int&  masterSize);


  int getstrand() const;

  void setstrand(int _strand);

  GeneR_seq * getGeneR_seq(int bufno);

  static GeneR_glob * instance();

 private:
  // Attributes
  /** \var max_buffers
   * \brief global variable of maximum buffer allowed
   * \note this variable can be increase using init_GeneR.
   */
  int max_buffers;

  /** \var myGeneR
   * \brief global variable containing all sequences buffers.
 * myGeneR[0] is an element of class GeneR_seq that contains
 * sequence, complementaire sequence and parameters sSize, sSizeComp
 * BegSeq and MasterSize
 */
  std::vector<GeneR_seq *> myGeneR;
/** \var strand
 * \brief global variable of current strand used
 */
  int strand;
  
  

};



#endif
