/*! \file GeneR_seq.cc
 *  \brief Complex functions for GeneR_seq class
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 19:48:01 antoine>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include "GeneR_seq.h"


/** @brief allocate a buffer
    @param size  Taille a allouer
    @param fill: 1. put a '\0' at the begining fill=0: don't fill
    @param comp  0 : brin sens -  1 : brin anti-sens
    Return 0: error; 1: OK
*/
int GeneR_seq::allocBuffer(int size, int comp, int fill)
{

  int rsize, csize;
  int tmp;
  char *b, *bb;

  
	
  rsize = (size_t) size;
	

	
  if (rsize <= 0)
    return(0);
	   
  if(comp)
    {
      csize = sSizeComp;
      b = seqComp;
    }
  else
    {
      csize = sSize;
      b = seq;
    }

  if (csize) {			/* realloc ?	*/
    if (rsize > csize)
	bb = (char*) realloc(b, rsize * sizeof(char));
    else {
      /* Nothing to alloc, all is already done */
      bb= b;
    }
  }
  else				/* malloc	*/
    bb = (char*) malloc(rsize * sizeof(char));	   

  if (bb == NULL)
    {
      printf("Memory allocation Error [GeneR]\n");
      return(0);
    }

  if(fill)
    bb[0] = '\0';
  //    (void) memset(bb, 'A', rsize);



  if(comp)
    {
      seqComp = bb;
      sSizeComp = rsize;
    }
  else
    {
      seq= bb;
      sSize = rsize;
    }

  tmp = sSize - 1;
  if (comp == 0)
    {
      sprintf(AccN,"%s","Seq_R");
      MasterSize = tmp;
      BegSeq = 1;
    }
  return(1);
}



// ------------------------------------
//
// Constructor
//
// initialize a sequence. (do not allocate 
// the sequence)
// ------------------------------------ 
GeneR_seq::GeneR_seq()
{
  seq = NULL;
  seqComp = NULL;
  sSize = 0;   /* size = length allocated = length of seq +1*/ 
  sSizeComp = 0;
  BegSeq = -1;
  MasterSize = -1;
  AccN[0] ='\0';
  //printf("c");
}


// ------------------------------------
//
// copy GeneR_seq from itself (just copy the structure;
// this function is called while using vector<GeneR_seq>)
//
//
// ------------------------------------
GeneR_seq::GeneR_seq(const GeneR_seq & tmpseq )
{
  seq = NULL;
  seqComp = NULL;
  sSize = 0;   /* size = length allocated = length of seq +1*/ 
  sSizeComp = 0;
  BegSeq = -1;
  MasterSize = -1;
  AccN[0] ='\0';
  //printf("r");
}


// ------------------------------------
//
// Destructor
//
// ------------------------------------
GeneR_seq::~GeneR_seq()
{
  if(seq != NULL)
    free(seq);
  if(seqComp != NULL)
    free(seqComp);

  //printf("o");
}

// ------------------------------------
//
// Free sequence and complementaire. Re init all values.
//
// ------------------------------------ 
void GeneR_seq::freeSeq()
{
      if(sSize!=0)
	{
	  free(seq);
	  free(seqComp);
	  seq = NULL;
	  seqComp = NULL;
	  sSize = 0;   /* size = length allocated = length of seq +1*/ 
	  sSizeComp = 0;
	  BegSeq = -1;
	  MasterSize = -1;
	  AccN[0] ='\0';
	}
}

// ------------------------------------
//
// Free sequence complementaire. Re init all values.
//
// ------------------------------------ 
void GeneR_seq::freeCompSeq()
{
  if(sSizeComp!=0)
    {
      free(seqComp);
      sSizeComp = 0;
      seqComp = NULL;
    }
}
