/*! \file GeneR_seq.h
 *  \brief sequence class for GeneR
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 19:36:25 antoine>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#ifndef GENER_SEQ_INCLUDED
#define GENER_SEQ_INCLUDED 1

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <string.h>
#include "GeneR_globals.h"
#include <iostream>


/** \class GeneR_seq 
 * \brief a class to manage one sequence and its complementaire
 */
class GeneR_seq
{

 private:
  // Attributes

  /** \var  seq  
   *  \brief sequences Here we store the ATGC...
   */
  char *seq;

  /** \var  seqComp  
   *  \brief reverse complementary sequences Here we store the GCAT...
   */
  char *seqComp;

  /** \var sSize
   * \brief size of sequence (allocated)
   */
  size_t  sSize;

  /** \var sSizeComp
   * \brief size of complemtentary sequence (allocated)
   */
  size_t  sSizeComp;

  /** \var BegSeq
   * \brief  begin position in source sequence (i.e. when  we load a part of
   * a sequence in a bank file, by example a subpart of a chromosome).
   */
  int BegSeq;

  /** \var MasterSize
   * \brief  size of source sequence (i.e. when  we load a part of
   * a sequence in a bank file, by example a subpart of a chromosome).
   */
  int MasterSize;
  /** \var AccN
   * \brief Name of sequence. By default: Seq_R
   */
  char AccN[MaxLNom];

 public:
 
  /** \brief Contructor: initialize a sequence. (do not allocate 
   * the sequence)*/
  GeneR_seq();

 /** \fn  GeneR_seq
  * \brief  copy GeneR_seq from itself (just copy the structure;
  * this function is called while using vector<GeneR_seq>)
  *
  */
  GeneR_seq(const GeneR_seq & tmpseq );

  /** \brief  Destructor */
  ~GeneR_seq();

  /** \fn freeSeq()
   * \brief Free sequence and complementaire. Re init all values.
   */
  void freeSeq();
  
  /** \fn freeCompSeq()
   * \brief Free sequence complementaire.
   */
  void freeCompSeq();
  
/** @brief allocate a buffer
    @param size  Size to allocate
    @param fill  Caractere de remplissage. fill=0: pas de remplissage
    @param comp  0 : brin sens -  1 : brin anti-sens
    Return 0: error; 1: OK
*/
  int allocBuffer(int size, int comp, int fill);


  // accesseurs
  inline char * getseq()
  {
    return seq;
  }
  inline char * getseqComp()
  {
    return seqComp;
  }
  inline size_t getsSize() const
  {
    return sSize;
  }
  inline size_t  getsSizeComp() const
  {
    return sSizeComp;
  }
  inline void setsSize(size_t _size) 
  {
    sSize = _size;
  }
  inline void  setsSizeComp(size_t _size)
  {
    sSizeComp = _size;
  }
  
  inline int  getBegSeq() const
  {
    return BegSeq;
  }
  inline void  setBegSeq(int _beg)
  {
    BegSeq = _beg;
  }

  inline int getMasterSize() const
  {
    return MasterSize;
  }

  inline void setMasterSize(int _size)
  {
    MasterSize = _size;
  }

  inline const char *  getAccN() const
  {
    return AccN;
  }

  inline void setAccN(char * _id)
  {
    if(strlen(_id) > MaxLNom)
      {
	std::cout << "GeneR::GeneR_seq::setAccN Ouch name "<<_id<<
	  "too long" << std::cout;
      }
    strncpy(AccN,_id,MaxLNom);
  }
};


#endif
