/*! \file dnaRna.cc
 *  \brief functions to change dna to rna and vice versa
 *
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 18:36:30 antoine>
 *  \author A. Lucas from "Bio-R" codes
 *  \note Licence: CeCIll
 */


#include <string>
#include "GeneR_glob.h"

extern "C"
{
  void dna_rna(int *bufno, int *vfrom,int *vto, int *comp, int *n);
  void rna_dna(int *bufno, int *vfrom,int *vto, int *comp, int *n);
}


/* ---------------------------------------------------- */
/*                      void dna_rna	                */
/* ---------------------------------------------------- */
/** @brief transforme les T en U dans la squence 
           contenue dans le bufseq bufno
    @param *bufno Numero de la sequence
    @param *vfrom Positions de debut des regions a transformer
    @param *vto   Positions de fin des regions a transformer
    @param *comp  0 : brin sens - 1 : brin anti-sens
    @param *n     Nombre de regions a transformer
*/  
/* ---------------------------------------------------- */

void dna_rna(int *bufno, int *vfrom,int *vto, int *comp, int *n)
{
  int i,k=0;
  char *seq;



  seq =  GeneR_glob::instance()->buffer(*bufno,0);
  if(seq == NULL)
    {
      *bufno = -1;
      return;
    } 

  while (k<*n)
    {
      if(*vto == 0)
	*vto=GeneR_glob::instance()->size(*bufno)-1;
      
      for (i=*vfrom-1;i<*vto;i++)
	{
	  if(seq[i]=='T')
	    seq[i]='U';
	  if(seq[i]=='t')
	    seq[i]='u';
	}
    
      *vfrom++;
      *vto++;
      k++;
    }
  return;	
}

/* ---------------------------------------------------- */
/*                        rna_dna	                */
/* ---------------------------------------------------- */
/** @brief transforme les U en T dans la squence
           contenue dans le bufseq bufno
    @param  *bufno Numero de la sequence
    @param  *vfrom Positions de debut des regions a transformer
    @param  *vto   Positions de fin des regions a transformer
    @param  *comp  0 : brin sens - 1 : brin anti-sens
    @param  *n     Nombre de regions a transformer
*/  
/* ---------------------------------------------------- */

void rna_dna(int *bufno, int *vfrom,int *vto, int *comp, int *n)
{
  int i,k=0;
  char *seq;


  seq =  GeneR_glob::instance()->buffer(*bufno,0);
  if(seq == NULL)
    {
      *bufno = -1;
      return;
    } 

  while (k<*n)
    {
      if(*vto == 0)
	*vto=GeneR_glob::instance()->size(*bufno)-1;
			
      for (i=*vfrom-1;i<*vto;i++)
	{
	  if(seq[i]=='U')
	    seq[i]='T';
	  if(seq[i]=='u')
	    seq[i]='t';
	}
    
      *vfrom++;
      *vto++;
      k++;
    }
  return;	
}

