/*! \file libIndex.cc
 * \brief some functions used when extract an index line.
 *
 *        An index line is composed of an Access number, the position
 *        of begining features, the position of the begining of sequence 
 *        and the size of sequence (size in file, it includes space and
 *        carriage return).
 *		                
 * \date created 8 Fevrier 2000	    
 * \date Last Modified : Time-stamp: <2006-01-16 10:59:03 lucas>
 * \note Copyright:  Copyright (c) Groupe Epissage CGM-Gif France
 */

#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include "libIndex.h"
#include "GeneR_globals.h"
#include <R.h>
#include <Rdefines.h>


namespace libIndex{


  /** \fn  ProchainMot(char *s, int pos, int Taille)
   * \brief   Renvoie la distance en caracteres du     
   *          debut du premier item suivant "pos"      */
  int ProchainMot(char *s, int pos, int Taille)
  {
    int i, Limite;
    
    Limite = Taille-pos;
    for(i=0; i < Limite && (isspace(s[i+pos])); i++);
    return i;
  } /* ProchainMot*/



  /** \fn LireMot(char *s, int pos, int Taille, char *Buf, int LimBuf)
   * @brief  Renvoie la taille de l'item debutant en "pos"       
   *   et le copie dans "Buf". Le termine par le caractere "\0"  
   */
  int LireMot(char *s, int pos, int Taille, char *Buf, int LimBuf)
  {
    int i, Limite;
    
    Limite = Taille-pos;
    for(i=0; i < Limite && i < (LimBuf-1) && (!isspace(s[i+pos])); i++)
      Buf[i] = s[i+pos];
    Buf[i] = '\0';
    return i;
  } /* LireMot*/





  /** \fn ExtrairePremierMot (char *Mot, char *Ligne)               
   * @brief  Extrait le premier mot d'une phrase       
   */
  int ExtrairePremierMot(char *Mot, char *Ligne)
  {
    int i = 0;
    
    while(!isspace(Mot[i] = Ligne[i])  ) 
      {
	if(Ligne[i] == '\0')
	  return(-1);
	i++;
      }
    Mot[i] = '\0';
    
    return 0;
  } /* ExtrairePremierMot*/



  /** \fn GetLigne (FILE *f, char *Ligne, int Pos)                          
   * @brief Retourne une ligne pointee sur un fichier  
   *  a partir de Pos jusqu'au prochain "\n"             
   * return -1 if error
   */
  int GetLigne(FILE *f, char *Ligne, int Pos)
  {



#ifndef __MINGW_H
    if(fseek(f, Pos, SEEK_SET) == -1)
      return(-1);
#else
    int i = 0;
    fseek(f, 0L, SEEK_SET);
    for(i=0; i< Pos; i++)
      if(fgetc(f) == EOF)
	return(-1);
    i=0;
#endif

    Ligne = fgets(Ligne,MaxLLigne,f);
    
    if(Ligne == NULL)
      return(-1);
  
    return 0;
  } /* GetLigne*/
  


  /** \fn GetTailleFic2(char *filename)
   *  \brief  Return size of file                         
   *  \param filename file name
   */
  
  int GetTailleFic2(char *filename)
  {
    struct stat inoeud;

    if(stat(filename, &inoeud) == -1)
      {
	fprintf(stderr,"Cannot get file size %s\n",filename);
	return -1;
      }
    return(inoeud.st_size);
  }



  /** \fn GetTailleLig( FILE *f)
   * @brief  Get size of first line of a file.
   *          file already open.
   * \param f the pointer to FILE
   */
  int GetTailleLig( FILE *f)
  {
    int i = 1;  /* pour inclure le '\n'*/
    while(fgetc(f) != '\n') i++;
    return i;
  } /* GetTailleLig */

  /** \fn returnInteger(int err)
   * \brief return an integer to R workspace
   * \param err integer to return
   */

  SEXP returnInteger(int err)
  {
    SEXP ans;
    PROTECT(ans = Rf_allocVector(INTSXP,1));
    INTEGER(ans)[0] = err;
    UNPROTECT(1);
    return ans;   
  }
  
}




