/*! \file misc.cc
 *  \brief Some uncategorised functions
 *  \version 1
 *  \date Created       : 10/02/05 
 *  \date Last Modified : Time-stamp: <2007-10-11 17:13:36 antoine>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include <string>
#include <R.h>
#include <Rdefines.h>
#include <stdio.h>

extern "C"
{
  SEXP str_to_nbstr (SEXP s,SEXP type) ;
  SEXP alloc_char(SEXP longR,SEXP nbR);
}

/** @brief Transform character string into number for sort
 *
 * @param s vector of strings
 * @param type integer (1, result is with integers in 0..255; 
 *                     2 result is in hexa; 3 result is binary.
 * @author A. Lucas
 */
SEXP str_to_nbstr (SEXP s,SEXP type) 
{
  SEXP ans;
  int i,j,letype,multiple,val;
  const char * mot;
  char * motnew;
  char * motnewbis;
  
  PROTECT (type = AS_INTEGER(type));
  letype = INTEGER(type)[0];
  UNPROTECT (1);

  /* letype : 
   * 1 le resultat doit tre sous la forme d'entiers entre 0 et 255
   * 2 le resultat doit tre en hexa
   * 3 le resultat doit tre en binaire
   */
  switch ( letype) {
    case 1:
	multiple = 3;
	break; 
    case 2:
 	multiple = 2;
	break; 
    case 3:
 	multiple = 8;
	break;  
  }


  switch (TYPEOF(s)) {
  case STRSXP:
    {
      PROTECT(ans = allocVector(STRSXP, LENGTH(s)));
      for (i = 0; i < LENGTH(s); ++i)
	{	  
	  mot = CHAR(STRING_ELT(s,i));
	  motnew = (char *) malloc((strlen(mot)*multiple+1)*sizeof(char));
	  for(j =0; j < (int)strlen(mot); j++)
	    {
	      switch (letype) {
	      case 1:
		sprintf(motnew+multiple*j,"%3.3d",mot[j]);
		break; 
	      case 2:
		sprintf(motnew+multiple*j,"%2.2x",mot[j]);
		break; 
	      }
	    }
	  SET_STRING_ELT(ans, i, mkChar(motnew));
	  free(motnew);
	}
      UNPROTECT(1);
      break;
    }
  case REALSXP:
    {
      printf("Not implemented for Real values\n");
      break;
    }
  case INTSXP:
  case LGLSXP:
    {
      PROTECT(ans = allocVector(STRSXP, LENGTH(s)));
      for (i = 0; i < LENGTH(s); ++i)
	{	  
	  val = INTEGER(s)[i];
	  motnewbis=(char *) malloc(25*sizeof(char));
	  sprintf(motnewbis,"%d",val);
	  motnew = (char *) malloc((multiple+1)*strlen(motnewbis)*sizeof(char));
	  switch (letype) {
	  case 1:
	    sprintf(motnew,"%3.3d",val);
	    break; 
	  case 2:
	    sprintf(motnew,"%2.2x",val);
	    break; 
	  case 3:
	    {
	      sprintf(motnewbis,"%o",val);
	      for (j =0; j< (int)strlen(motnewbis) ; j++)
		switch (motnewbis[j]){
		case '0':
		    motnew[j*3]='0';
		    motnew[j*3+1]='0';
		    motnew[j*3+2]='0';
		    break; 
		case '1':
		    motnew[j*3]='0';
		    motnew[j*3+1]='0';
		    motnew[j*3+2]='1';
		    break; 
		case '2':
		    motnew[j*3]='0';
		    motnew[j*3+1]='1';
		    motnew[j*3+2]='0';
		    break; 
		case '3':
		    motnew[j*3]='0';
		    motnew[j*3+1]='1';
		    motnew[j*3+2]='1';
		    break; 
		case '4':
		    motnew[j*3]='1';
		    motnew[j*3+1]='0';
		    motnew[j*3+2]='0';
		    break; 
		case '5':
		    motnew[j*3]='1';
		    motnew[j*3+1]='0';
		    motnew[j*3+2]='1';
		    break; 
		case '6':
		    motnew[j*3]='1';
		    motnew[j*3+1]='1';
		    motnew[j*3+2]='0';
		    break; 
		case '7':
		    motnew[j*3]='1';
		    motnew[j*3+1]='1';
		    motnew[j*3+2]='1';
		    break; 
		}
	      motnew[j*3]='\0';
	      break; 
	    }
	  case 4:
	    sprintf(motnew,"%8.8o",val);
	    break; 
	    
	  }
	  SET_STRING_ELT(ans, i, mkChar(motnew));
	  free(motnew);
	}
      UNPROTECT(1);
      break;
    }
  }
  return ans;
}



/** \brief this function allocate nb character string of
 * size long, and return vector of char to R. This function
 * is use to allocate everything in R (double and integer:
 * use function integer(nb), double(nb) so that we could
 * [partly] avoid theses functions SEXP... and use only
 * .C call.
 * \author A. Lucas
 */
SEXP alloc_char(SEXP longR,SEXP nbR)
{
  SEXP ans;
  char * tmp;
  int longC,nbC,i;

  longC =  INTEGER(longR)[0] +1;
  nbC =  INTEGER(nbR)[0];
  tmp = (char *) malloc(longC * sizeof(char) );

  for(i = 0 ; i<longC; i++)
    tmp[i]='x';
  tmp[longC-1]='\0';

  PROTECT(ans = allocVector(STRSXP, nbC));
  for(i =0; i<nbC; i++)
    SET_STRING_ELT(ans, i, mkChar(tmp));
  UNPROTECT(1);

  free(tmp);
  return(ans);
}
