/*! \file readEmblDescript.cc
 *  \brief read features in an EMBL file
 *  \version 1
 *  \date Created       : 21/01/06 
 *  \date Last Modified : Time-stamp: <2007-10-11 17:10:03 antoine>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <vector>

#include "libIndex.h"
#include "GeneR_globals.h"

#include <R.h>
#include <Rdefines.h>


extern "C"
{

  SEXP readEmblDescript(SEXP file ,SEXP  debfeature, SEXP debseq,
			SEXP code);
}



using namespace std;
/** \fn readLocation
 * \brief read Embl feature, identified by two letters "code"
 *
 */ 

SEXP readEmblDescript(SEXP file ,SEXP  debfeatures, SEXP debseq,
		      SEXP code)
{

  SEXP ans;

  const char * fileC;
  const char * codeC;
  int debfeaturesC;
  int debseqC;
  int i=0;
  int j=0; /* number of vectors */

  string OneLine;
  vector<string> result (1);

  /* Get inputs from R */
  fileC = CHAR(STRING_ELT(file,0));
  codeC = CHAR(STRING_ELT(code,0));

  debfeaturesC = INTEGER(debfeatures)[0]; /* Must be first column of index file */
  debseqC = INTEGER(debseq)[0];
  
  

  ifstream thefichier (fileC,ios::in);
  if(!thefichier)
    {
      cout << "GeneR.so: open file error";
      return(libIndex::returnInteger(0));
    }

  thefichier.seekg(debfeaturesC,ios::beg);
  if(!thefichier)
    {
      cout << "GeneR.so: seek in file error";
      thefichier.close();
      return(libIndex::returnInteger(0));
    }
  getline(thefichier,OneLine,'\n');
  

  while(thefichier)
    {
      getline(thefichier,OneLine,'\n');
      if(OneLine.find(codeC)==0)
	{
	  /* add a space between lines */
	  if(i>0)
	    result[j] += " ";

	  result[j] += OneLine.substr(5,OneLine.size());
	  i++;
	}
      else      
	if(i >0)
	  {
	    j++;
	    i=0;
	    result.push_back("");
	  }

      
      if(thefichier.tellg() > debseqC)
	break;
    }


  thefichier.close();

  if(i!=0)
    j++;

  PROTECT(ans = allocVector(STRSXP, j));

  for(i =0; i<j; i++)
    SET_STRING_ELT(ans, i, mkChar( result[i].c_str() ));

  UNPROTECT(1);


  return(ans);

}
