/*! \file readIndex.cc
 *  \brief functions to read entries in index
 *
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2006-01-16 11:03:14 lucas>
 *  \author A. Lucas from "Bio-R" codes
 *  \note Licence: CeCIll
 */

#include <stdio.h>
#include <ctype.h>
#include <string>
#include "readIndex.h"
#include "readSeqEmbl.h"
#include "readSeqGbk.h"
#include "libIndex.h"
#include "libStrings.h"
#include "GeneR_globals.h"


extern "C"
{
  void XtLigIxC (char **accn,char **indexfile,int *offsfeat,int *offseq,int *size);
  void sizeseqemblC (char **accn, int *offsetseq, char **bankpath);
  void sizeseqgbk (char **accn, int *offsetseq, char **bankpath);
  void sizeseqfasta (char **accn, int *offsetseq, char **bankpath);

}



/* ---------------------------------------------------- */
/*          XtLigIxR (char **accn,char **indexfile)	*/
/* ---------------------------------------------------- */
/** @brief cherche dans le fichier indexfile
           les parametres suivants correspondant
	    l'accn donn :	
	   offset features, offset sequence, 
	   la taille de l'entree
    @param **accn      Numero d'accession ou d'identite
    @param **indexfile Chemin d'acces au fichier index
    @param *offsfeat   Offset features. Valeur de retour dans R.
    @param *offseq     Offset sequence. Valeur de retour dans R.
    @param *size       Taille de l'entree. Valeur de retour dans R.
*/
/* ---------------------------------------------------- */
     
void XtLigIxC (char **accn,char **indexfile,int *offsfeat,int *offseq,int *size)
{
  char *res;
  char nomacc [MaxLNom] = "";


  if(readIndex::XtLigIx(*accn,*indexfile,&res) == -1)
    {
      *size = -1;            /* valeur d'erreur dans R */
      return ;
    }

  readIndex::SplitIxLine(res,nomacc,offsfeat,offseq,size);
  free(res);
}


/* ------------------------------------------------------- */
/*                      sizeseqembl                        */
/* ------------------------------------------------------- */
/** @brief retourne la longueur de la sequence dont
           l'accn est rpertori dans indexfile
	   < LC > add 25 May 00
    @param **accn     Numero d'accessio nou d'identite de la sequence
    @param *offsetseq  Offset sequence
    @param **bankpath Chemin d'acces du fichier banque
*/
/* ------------------------------------------------------- */

void sizeseqemblC (char **accn, int *offsetseq, char **bankpath)
{
  FILE *fic;
  char * currentline;
  char lastline[MaxLLigne];

  currentline = (char *) malloc(MaxLLigne * sizeof(char));

  if ( !(fic = fopen(*bankpath, "r")))
    {
      printf("GeneR.so: error while opening file %s\n",*bankpath);
      *offsetseq = -1;          /* valeur d'erreur dans R */
      return;
    }
#ifndef __MINGW_H
  if (fseek(fic, *offsetseq,0))
    {
      printf("GeneR.so: error while seeking file %s\n",*bankpath);
      *offsetseq = -1; 
      fclose(fic);
      return;
    }
#else
  int i;
  /* fseek on windows count CR ('\r') and fgets does not see them */
  for(i=0; i< *offsetseq; i++)
    if(fgetc(fic) == EOF)
      {
	printf("GeneR.so: error while seeking file\n");
	*offsetseq = -1; 
	fclose(fic);
	return ;
      }
#endif

  currentline =fgets(currentline, MaxLLigne, fic);
  if (readSeqEmbl::EstFinEntreeEMBL(currentline))
    {
      *offsetseq = -1; 
      return ;	/* pas de sequence */
    }

  while (!(readSeqEmbl::EstFinEntreeEMBL(currentline)))
    {
      /* Si la ligne n'est pas vide*/
      if(strlen(currentline)> 2)
	memcpy(lastline,currentline,MaxLLigne);

      currentline =fgets(currentline, MaxLLigne, fic);
     }
	
  *offsetseq = readSeqEmbl::XtNumntFinLig(lastline,1);     /* valeur de retour dans R */

  fclose(fic);

  return;
}


/* ------------------------------------------------------- */
/* @Function: sizeseqgbk (char *accn, int *offsetseq,     */
/* @+			   char *bankpath)		   */
/** \brief Size of a sequence in a Genbank file
 *
 *  Return size of the sequence 'accn' 
 retourne la longueur de la sequence dont 	   
 @+         l'accn est rpertori dans indexfile	   	
    @param **accn     Numero d'accessio nou d'identite de la sequence
    @param *offsetseq  Offset sequence
    @param **bankpath genebank file  */

/* ------------------------------------------------------- */

void sizeseqgbk (char **accn, int *offsetseq, char **bankpath)
{
  FILE *fic;
  char currentline[MaxLLigne];
  char lastline[MaxLLigne];
  char lastline2[MaxLLigne];
  char endseq[MaxLLigne];
  int i, l, j,k;
  int lastNbre;

  if ( !(fic = fopen(*bankpath, "r")))
    {
      printf("GeneR.so: error while opening file %s\n",*bankpath);
      *offsetseq = -1;          /* valeur d'erreur dans R */
      return;
    }

#ifndef __MINGW_H
  if (fseek(fic, *offsetseq,0))
    {
      printf("GeneR.so: error while seeking file %s\n",*bankpath);
      *offsetseq = -1; 
      fclose(fic);
      return;
    }
#else
  /* fseek on windows count CR ('\r') and fgets does not see them */
  for(i=0; i< *offsetseq; i++)
    if(fgetc(fic) == EOF)
      {
	printf("error while seeking file\n");
	*offsetseq = -1; 
	fclose(fic);
	return ;
      }
#endif

  fgets(currentline, MaxLLigne, fic);
  if (readSeqGbk::EstFinEntreebk(currentline))
    {
      *offsetseq = -1; 
      fclose(fic);
      return ;	/* pas de sequence */
    }

  while (!(readSeqGbk::EstFinEntreebk(currentline)))
    {
      strcpy(lastline,currentline);
      fgets(currentline, MaxLLigne, fic);
    }

  lastNbre = readSeqGbk::XtNumntDebLig(lastline);

  libStrings::SupprimerSeparateurs(lastline, lastline2);
 
  i=0;
 
  while(isdigit(lastline2[i]))   /*  libgenbank.c */
    i++;  

  l = strlen(lastline2);
 
  for(j=0,k=i;k<=l;j++,k++)
    endseq[j] = lastline2[k];
 
  *offsetseq  = lastNbre + strlen(endseq) - 1 ;  

  fclose(fic);

  return;
}




namespace readIndex
{
  /* ---------------------------------------------------- */
  /* @Function: SplitIxLine	                        */
  /** @brief  Ventile une ligne d'index       */
  /* ---------------------------------------------------- */
  int SplitIxLine (char *Lig, char *NomAcc, int *OffsFT,
                 int *OffsSQ, int *TailleEntree) 
  {
    int taille, pos;	/* longueur de la ligne & position du curseur*/
    char ch[MaxLLigne];
    
    taille = strlen(Lig);
    pos = 0;
    libIndex::LireMot(Lig, pos, taille, NomAcc, MaxLLigne);
    pos += strlen(NomAcc);
    pos += libIndex::ProchainMot(Lig, pos, taille);
    libIndex::LireMot(Lig, pos, taille, ch, MaxLLigne);
    sscanf(ch, "%d", OffsFT);
    pos += strlen(ch);
    pos += libIndex::ProchainMot(Lig, pos, taille);
    libIndex::LireMot(Lig, pos, taille, ch, MaxLLigne);
    sscanf(ch, "%d", OffsSQ);
    pos += strlen(ch);
    pos += libIndex::ProchainMot(Lig, pos, taille);
    libIndex::LireMot(Lig, pos, taille, ch, MaxLLigne);
    sscanf(ch, "%d", TailleEntree);
    return 0;
  } /* SplitIxLine */


  /* ---------------------------------------------------- */
  /* @Function: XtLigIx                                   */
  /** @brief  Extraction d'une ligne d'index            
   *         Arguments AccNumber /path/NomFichIndex	*/
  /* ---------------------------------------------------- */
  int XtLigIx(char *NomSeq, char *NomIndex, char **Res)
  {
    FILE *f;                      /* fichier de travail*/
    int TailleFic;               /* Taille du fichier d'index*/
    int NbcpL;                    /* nb lignes & car du fichier d'index*/
    int NbLigF;                  /* Nbre de lignes du fichier d'index*/
    /*char *Lig;*/                    /* La ligne d'index courante*/

  
    /* taille du fichier*/

    TailleFic = libIndex::GetTailleFic2(NomIndex);


    /* ouvrir le fichier et evaluer la longueur d'une ligne*/
    if( (f = fopen(NomIndex, "r")) == NULL)
      return -1;
		

    NbcpL = libIndex::GetTailleLig (f);
    if(NbcpL<42)
      NbcpL= 42;
    /*  printf("Nb ling %d",NbcpL);*/
    /*Lig = (char *) malloc ((NbcpL +1)*sizeof(char));*/
    *Res = (char *) malloc ((NbcpL +1)*sizeof(char));
  
    NbLigF = TailleFic / NbcpL;

    /* Appeler la procedure de recherche*/
    if(Rech_dicho (f, NomSeq,*Res , NbLigF, NbcpL) == -1)
  	{
	  fclose(f);
	  free(*Res);
	  return -1;
	}

    /* Fermer le fichier d'index*/
    fclose(f);

    /* Note: free(*Res) is call later when return = 0 */   

    return 0;

  } /* XtLigIx*/



  /* ---------------------------------------------------- */
  /* @Function: Rech_dicho				*/
  /** @brief  Recherche dichotomique dans un fichier	
   * du premmier mot d'une ligne. Renvoie la ligne si trouvee	
   * ou une chaine vide dans Ligne. Retourne 0 si trouve, -1 sinon.	*/
  /* ---------------------------------------------------- */
  int Rech_dicho (FILE *f, char *Cible, char *Ligne, int NbL, int NbcpL)
  {
    int mediane, gauche = 0, droite = NbL - 1;
    char Nom[MaxLNom];
    int Pos;
    int Res;
    /*int i;*/			/* pour tester*/
    
    /* si cible <= premiere valeur, trop petit. Retourner chaine vide*/

    if(libIndex::GetLigne(f, Ligne, gauche) == -1)
      return(-1);


    libIndex::ExtrairePremierMot(Nom, Ligne);

    /*  printf("Premiere ligne >%s< mot >%s<\n", Ligne, Nom);*/
    Res = strcmp(Cible, Nom);
    if(Res < 0) {
      Ligne[0] = '\0';
      return(-1);
    }
    else if (Res == 0) return 0; /* la ligne est la premiere*/
    

    /* si cible > derniere valeur, trop grand, retourner chaine vide */
    Pos = droite * (long) NbcpL;
    if(libIndex::GetLigne(f, Ligne, Pos) == -1)
      return(-1);

    libIndex::ExtrairePremierMot(Nom, Ligne);
    /*  printf("Pos: %ld Derniere ligne >%s< mot >%s<\n", Pos, Ligne, Nom); */
    Res = strcmp(Cible, Nom);
    /*printf("Res= %d\n", Res);*/
    if(Res > 0) {
      Ligne[0] = '\0';
      return -1;
    }
    else if (Res == 0) return 0; /* la ligne est la derniere*/
    
    /*i = 0;*/ /* test*/
    while (droite - gauche > 1)
      {
	mediane = (droite + gauche) / 2;
	Pos = mediane * (long) NbcpL;
	if(libIndex::GetLigne(f, Ligne, Pos) == -1)
	  return(-1);
	libIndex::ExtrairePremierMot(Nom, Ligne);
	Res = strcmp(Cible,Nom);
	if (Res < 0) droite = mediane;
	else if (Res > 0) gauche = mediane; 
	else if (Res == 0) droite = gauche;
      } /* while*/

    
    if (Res == 0) return 0 ;
    else {
      Ligne[0] = '\0';
      return -1;
    } /* else*/
    
  } /* Rech_dicho*/



}


/* ------------------------------------------------------- */
/* @Function: sizeseqfasta (char *accn, int *offsetseq,     */
/* @+			   char *bankpath)		   */
/** @brief  retourne la longueur de la sequence dont 	   
         l'accn est rpertori dans indexfile	   	
	 @date < LC > add 25 May 00			   	
 * ------------------------------------------------------- */

void sizeseqfasta (char **accn, int *offsetseq, char **bankpath)
{
  FILE *fic;

  int i = 0;
  char c;
  
  if ( !(fic = fopen(*bankpath, "r")))
    {
      printf("GeneR.so: error while opening file %s\n",*bankpath);
      *offsetseq = -1;          /* valeur d'erreur dans R */
      return;
    }

#ifndef __MINGW_H
  if (fseek(fic, *offsetseq,0))
    {
      printf("GeneR.so: error while seeking into file %s\n",*bankpath);
      *offsetseq = -1; 
      fclose(fic);
      return;
    }
#else
  /* fseek on windows count CR ('\r') and fgets does not see them */
  for(i=0; i< *offsetseq; i++)
    if(fgetc(fic) == EOF)
      {
	printf("GeneR.so: error while seeking file\n");
	fclose(fic);
	*offsetseq = -1;
	return ;
      }
  i=0;
#endif

  c=fgetc(fic);
  while(c != '>' && c != EOF)
    {
      if(c != '\n')
	i++;
      c=fgetc(fic); 
    }
  
  *offsetseq = i;

  fclose(fic);

  return;
}
