/*! \file upperSeq.cc
 *  \brief functions to put in lower / upper case regions of sequences.
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 18:43:12 antoine>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include <string>
#include "GeneR_glob.h"


extern "C"
{
  void  upper_buffer (int * seqno, int * from , int * to,int * reverse,
		      int * long_from);
  void  lower_buffer (int * seqno, int * from , int * to,int * reverse,
		      int * long_from);

}



#define TO_LOWER(c) ((c) - 'A' + 'a')
#define IS_LOWER(c) (((c) >= 'a') && ((c) <= 'z'))
#define TO_UPPER(c) ((c) - 'a' + 'A')
#define IS_UPPER(c) (((c) >= 'A') && ((c) <= 'Z'))
/** Upper_buffer put regions of seqno in upper case
 * \brief from and to defines regions that will be in upper case
 * \param seqno: integer, buffer number
 * \param from, to: vectors, define regions (from < to)
 * \param reverse: flag, if 0: we work on buffer forward, else we work on
 *                 the reverse. Adresses of from and to are "transformed" in R.
 * \param long_from: size of from and to.
 * \author A. Lucas
 */
void  upper_buffer (int * seqno, int * from , int * to,int * reverse,int * long_from)
{
  int i,j;
  char * le_buffer;

  /* from = 1: premiere position ds le buffer... i.e. buffer[0] */


  le_buffer=GeneR_glob::instance()->buffer(* seqno,reverse[0]);
  if(!le_buffer)
    {
      *seqno=-1;
      return;
    }

  for(i=0; i< long_from[0]; i++)
    for(j=from[i]-1; j< to[i]; j++)      
      {
	if(IS_LOWER(le_buffer[j]))
	  le_buffer[j] = TO_UPPER(le_buffer[j]);
      }
}

/** Lower_buffer put regions of seqno in lower case
 * \brief from and to defines regions that will be in lower case
 * \param seqno: integer, buffer number
 * \param from, to: vectors, define regions (from < to)
 * \param reverse: flag, if 0: we work on buffer forward, else we work on
 *                 the reverse. Adresses of from and to are "transformed" in R.
 * \param long_from: size of from and to.
 * \author A. Lucas
 */
void  lower_buffer (int * seqno, int * from , int * to,int * reverse,int * long_from)
{
  int i,j;
  char * le_buffer;

  /* from = 1: premiere position ds le buffer... i.e. buffer[0] */

  le_buffer=GeneR_glob::instance()->buffer(* seqno,reverse[0]);
  if(!le_buffer)
    {
      *seqno=-1;
      return;
    }


  for(i=0; i< long_from[0]; i++)
    for(j=from[i]-1; j< to[i]; j++)      
      {
	if(IS_UPPER(le_buffer[j]))
	  le_buffer[j] = TO_LOWER(le_buffer[j]);
      }

}
#undef TO_UPPER
#undef TO_LOWER
#undef IS_UPPER
#undef IS_LOWER


