/*! \file writeFasta.cc
 *  \brief functions to write Fasta file
 *
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 18:44:04 antoine>
 *  \author A. Lucas from "Bio-R" codes
 *  \note Licence: CeCIll
 */

#include <stdio.h>
#include <string>
#include "GeneR_glob.h"


extern "C"
{
  void sys_write_fasta_seq (int *bufseq, int *from, int *to, char **name, 
			    char **comment, char **file, int *comp,
			    int *char_per_line, int *n,
			    int* err);
  void write_fasta_seq (char ** Buffer, int *from, int *to, char **name, 
			char **comment, char **file, 
		        int *char_per_line, int *n,
			int * err);
}

/* ----------------------------------------------------------------------- */ 
/*                           sys_write_fasta_seq                           */
/* ----------------------------------------------------------------------- */ 
/** @brief ecrit une  portion de la sequence 
           en format fasta dans un fichier
	   fortement inspir par la fonction WriteFastaSequence de la
	   librairie libfasta de l'ABI (cf abisio)
	   [LC] created 22 Oct 00
    @param bufseq        Numero de la sequence
    @param from          Positions de debut des regions a ecrire
    @param to            Positions de fin des regions a ecrire
    @param name        Nom de la sequence
    @param comment     Commentaire additionnel
    @param file        Chemin d'acces du fichier de sortie
    @param comp          0 : brin sens. 1: brin anti-sens
    @param char_per_line Nombre de caracteres par ligne 
                              dans le fichier fasta
    @param n         Nombre de regions a ecrire
    @param err        erreur: -1 lorsque sequence ou fichier non accessible    
*/
void sys_write_fasta_seq (int *bufseq, int *from, int *to, char **name, 
			char **comment, char **file, int *comp,
		        int *char_per_line, int *n,
			   int * err)
{
  char * Buffer;
  Buffer =  GeneR_glob::instance()->buffer(*bufseq,*comp );

  if (! Buffer)
    {
      *err = -1;
      return;
    }

  write_fasta_seq (&Buffer,  from, to,name, comment, file,
		   char_per_line, n,err);


}



void write_fasta_seq (char ** Buffer, int *from, int *to, char **name, 
			char **comment, char **file, 
		        int *char_per_line, int *n,int * err)
{
  FILE *f;
  int i,j, length, nlines, rest;
  char *buf, *end, tempo;


 
  if( !(f = fopen(*file,"a")))
    {
      *err = -1;
      return;
    }


  j=0;
  while(*n>0)
    {
      (void) fputc('>', f);
      (void) fputs((name)[j],f);
      (void) fputc(' ',f);
      (void) fputs((comment)[j],f);
      (void) fputc('\n',f);

      length = *to - *from + 1;
      
      nlines = length / *char_per_line;

      buf = *Buffer + *from - 1;

      for (i = 0 ; i < nlines ; i++) {
	end = buf + *char_per_line;
	tempo = *end;
	*end = '\000';
	(void) fputs(buf, f);
	(void) fputc('\n' , f);
	*end = tempo;
	buf += *char_per_line;
      }

      if ((rest = (length % *char_per_line)) != 0) {
	end = buf + rest;
	tempo = *end;
	*end = '\000';
	(void) fputs(buf, f);
	(void) fputc('\n', f);
	*end = tempo;
      }
      from++;to++;*n=*n-1;j++;
    }
  fclose(f);
}

